package samples.files;

/*
 * A simple application that demonstrates the use of Aspera Java fasp Manager
 * API. Aspera, Inc.
 */

import java.util.List;
import java.util.concurrent.CountDownLatch;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.asperasoft.faspmanager.Encryption;
import com.asperasoft.faspmanager.FaspManager;
import com.asperasoft.faspmanager.FaspManagerException;
import com.asperasoft.faspmanager.FileInfo;
import com.asperasoft.faspmanager.LocalLocation;
import com.asperasoft.faspmanager.Policy;
import com.asperasoft.faspmanager.RemoteLocation;
import com.asperasoft.faspmanager.Resume;
import com.asperasoft.faspmanager.SessionStats;
import com.asperasoft.faspmanager.TransferEvent;
import com.asperasoft.faspmanager.TransferListener;
import com.asperasoft.faspmanager.TransferOrder;
import com.asperasoft.faspmanager.XferParams;

public class SimpleParallelFileTransfer {

	protected static final Logger	log	= LoggerFactory.getLogger(SimpleParallelFileTransfer.class);

	private static CountDownLatch	latch;

	private static class ParallelTransferListener implements TransferListener {

		@Override
		public void fileSessionEvent(TransferEvent event, SessionStats sessionStats, FileInfo fileInfo) {
			// The xferId can be used to correlate multiple sessions belonging to the same transfer.
			log.info("xferId={}", sessionStats.getXferId());

			switch (event) {
			case PROGRESS:
				log.info("Progress: sessionId={} {}%", sessionStats.getSessionId(), (sessionStats.getTotalTransferredBytes() * 100)
						/ sessionStats.getPreCalcTotalBytes());
				break;
			case SESSION_STOP:
				log.info("session ended: jobId={}", sessionStats.getId());
				latch.countDown();
				break;
			case SESSION_ERROR:
				log.error("Session Error: jobId={} error={}", sessionStats.getId(), sessionStats.getErrorDescription());
				break;
			default:
			}
		}

	}

	private static final int	NB_PROCESSES	= 4;

	public static void main(String[] args) throws Exception {
		try {

			/*
			 * A listener added to the fasp Manager(as below) will be notified on all events and on all jobs. It is also possible to
			 * add a listener to only a particular job of interest.
			 */
			FaspManager.getSingleton().addListener(new ParallelTransferListener());

			/*
			 * Start a download. Make sure the destination directory exists (in the example below the destination directory is ".").
			 */
			System.out.println("Starting download...");
			latch = new CountDownLatch(NB_PROCESSES);

			startDownload("demo.asperasoft.com", "aspera", "demoaspera", new String[] { "aspera-test-dir-large/100MB" }, "./âspëra-100MB",
					NB_PROCESSES);

			/*
			 * Wait for the download to finish.
			 */
			latch.await();
			System.out.println("Download completed!");

			System.out.println("Starting upload...");
			latch = new CountDownLatch(NB_PROCESSES);

			startUpload("demo.asperasoft.com", "asperaweb", "demoaspera", new String[] { "âspëra-100MB" }, "Upload", NB_PROCESSES);

			/*
			 * Wait for the upload to finish.
			 */
			latch.await();
			System.out.println("Upload completed!");

			FaspManager.destroy();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	/*
	 * Set the transfer parameters for this transfer. Ideally, the target rate is equal to the bandwidth capacity of the Aspera
	 * computer (in kilo bits per second). For example, for a T1 link set targetRate = 1500 (1.5 mega bits per second). fasp will never
	 * exceed the specified target rate.
	 */
	static XferParams myTransferParams() {
		XferParams p = new XferParams();
		/* For most cases, these port numbers need not be changed */
		p.tcpPort = 22;
		p.udpPort = 33001;
		p.encryption = Encryption.NONE;
		// Policy.ADAPTIVE: fasp will try to match the target rate while being
		// fair to other traffic on the network
		p.policy = Policy.FAIR;
		// p.cookie = "FaspManagerSample1-Cookie";
		// With newer versions of ascp if a token is passed it must be valid
		// regardles of whether or not username/password authentication is used
		// p.token = "FaspManagerSample1-Token";
		// Resume.OFF will not resume the transfer after a broken transfer
		p.resumeCheck = Resume.OFF;
		p.preCalculateJobSize = true;
		return p;
	}

	/*
	 * Sets up and submits a download job.
	 */
	static List<String> startDownload(String host, String user, String password, String[] remoteSourcePaths, String localDestinationPath,
			int nbProcesses) throws FaspManagerException {

		/* Create a remote source location object */
		RemoteLocation remoteSourceFiles = new RemoteLocation(host, user, password);

		/* Add paths on the remote source to the source object */
		for (String path : remoteSourcePaths)
			remoteSourceFiles.addPath(path);

		/* Create a local destination object */
		LocalLocation localDestination = new LocalLocation();

		/* Add path to the destination object */
		localDestination.addPath(localDestinationPath);

		/*
		 * Setup a job order. A job order consists of source, destination and the transfer params(XferParams object)
		 */
		TransferOrder order = new TransferOrder(remoteSourceFiles, localDestination, myTransferParams());

		/* Submit the job for transfer */
		return FaspManager.getSingleton().startTransfer(order, nbProcesses);
	}

	/*
	 * Sets up and submits an upload job.
	 */
	static List<String> startUpload(String host, String user, String password, String[] localSourcePaths, String remoteDestinationPath,
			int nbProcesses) throws FaspManagerException {

		/* Create a local source location object */
		LocalLocation localSourceFiles = new LocalLocation();

		/* Add local paths to the source object */
		for (String path : localSourcePaths)
			localSourceFiles.addPath(path);

		/* Create a remote destination object */
		RemoteLocation remoteDestination = new RemoteLocation(host, user, password);

		/* Add remote path to the destination object */
		remoteDestination.addPath(remoteDestinationPath);

		/*
		 * Setup a job order. A job order consists of source, destination and the transfer params(XferParams object)
		 */
		TransferOrder order = new TransferOrder(localSourceFiles, remoteDestination, myTransferParams());

		/* Submit the job for transfer */
		return FaspManager.getSingleton().startTransfer(order, nbProcesses);
	}

}
